/* enum name checklist, for libreswan
 *
 * Copyright (C) 2023-2024  Andrew Cagney
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <https://www.gnu.org/licenses/gpl2.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include "constants.h" /* for typedef ${names} */
#include "enum_names.h"

#include "connection_owner.h"
#include "encap_mode.h"
#include "encap_proto.h"
#include "encap_type.h"
#include "fips_mode.h"
#include "ietf_constants.h"
#include "initiated_by.h"
#include "kernel_mode.h"
#include "message_role.h"
#include "names_constant.h"
#include "pluto_constants.h"
#include "sa_role.h"
#include "sa_type.h"
#include "state_category.h"

const struct enum_names_check enum_names_checklist[] = {
  { "ah_transformid_names", &ah_transformid_names, },
  { "allow_global_redirect_names", &allow_global_redirect_names, },
  { "attr_msg_type_names", &attr_msg_type_names, },
  { "auth_alg_names", &auth_alg_names, },
  { "certpolicy_type_names", &certpolicy_type_names, },
  { "connection_event_kind_names", &connection_event_kind_names, },
  { "connection_kind_names", &connection_kind_names, },
  { "connection_owner_names", &connection_owner_names, },
  { "connection_owner_stories", &connection_owner_stories, },
  { "dns_auth_level_names", &dns_auth_level_names, },
  { "doi_names", &doi_names, },
  { "eap_code_names", &eap_code_names, },
  { "eap_type_names", &eap_type_names, },
  { "eaptls_flag_names", &eaptls_flag_names, },
  { "encap_mode_names", &encap_mode_names, },
  { "encap_mode_story", &encap_mode_story, },
  { "encap_proto_names", &encap_proto_names, },
  { "encap_proto_story", &encap_proto_story, },
  { "encap_type_names", &encap_type_names, },
  { "encapsulation_mode_names", &encapsulation_mode_names, },
  { "esp_transformid_names", &esp_transformid_names, },
  { "event_type_names", &event_type_names, },
  { "fips_mode_names", &fips_mode_names, },
  { "global_timer_names", &global_timer_names, },
  { "ike_cert_type_names", &ike_cert_type_names, },
  { "ike_id_type_names", &ike_id_type_names, },
  { "ike_version_names", &ike_version_names, },
  { "ikev1_exchange_names", &ikev1_exchange_names, },
  { "ikev1_ike_id_type_names", &ikev1_ike_id_type_names, },
  { "ikev1_ipsec_attr_names", &ikev1_ipsec_attr_names, },
  { "ikev1_payload_names", &ikev1_payload_names, },
  { "ikev1_protocol_names", &ikev1_protocol_names, },
  { "ikev1_xauth_type_names", &ikev1_xauth_type_names, },
  { "ikev2_auth_method_names", &ikev2_auth_method_names, },
  { "ikev2_cert_type_names", &ikev2_cert_type_names, },
  { "ikev2_cp_attribute_type_names", &ikev2_cp_attribute_type_names, },
  { "ikev2_cp_type_names", &ikev2_cp_type_names, },
  { "ikev2_delete_protocol_id_names", &ikev2_delete_protocol_id_names, },
  { "ikev2_exchange_names", &ikev2_exchange_names, },
  { "ikev2_hash_algorithm_names", &ikev2_hash_algorithm_names, },
  { "ikev2_ike_id_type_names", &ikev2_ike_id_type_names, },
  { "ikev2_last_proposal_desc", &ikev2_last_proposal_desc, },
  { "ikev2_last_transform_desc", &ikev2_last_transform_desc, },
  { "ikev2_notify_protocol_id_names", &ikev2_notify_protocol_id_names, },
  { "ikev2_payload_names", &ikev2_payload_names, },
  { "ikev2_ppk_id_type_names", &ikev2_ppk_id_type_names, },
  { "ikev2_proposal_protocol_id_names", &ikev2_proposal_protocol_id_names, },
  { "ikev2_redirect_gw_names", &ikev2_redirect_gw_names, },
  { "ikev2_trans_attr_descs", &ikev2_trans_attr_descs, },
  { "ikev2_trans_type_encr_names", &ikev2_trans_type_encr_names, },
  { "ikev2_trans_type_esn_names", &ikev2_trans_type_esn_names, },
  { "ikev2_trans_type_integ_names", &ikev2_trans_type_integ_names, },
  { "ikev2_trans_type_names", &ikev2_trans_type_names, },
  { "ikev2_trans_type_prf_names", &ikev2_trans_type_prf_names, },
  { "ikev2_ts_type_names", &ikev2_ts_type_names, },
  { "initiated_by_names", &initiated_by_names, },
  { "ip_protocol_id_names", &ip_protocol_id_names, },
  { "ipsec_attr_names", &ipsec_attr_names, },
  { "ipsec_ipcomp_algo_names", &ipsec_ipcomp_algo_names, },
  { "ipseckey_algorithm_config_names", &ipseckey_algorithm_config_names, },
  { "ipseckey_algorithm_type_names", &ipseckey_algorithm_type_names, },
  { "isakmp_flag_names", &isakmp_flag_names, },
  { "isakmp_transformid_names", &isakmp_transformid_names, },
  { "isakmp_xchg_type_names", &isakmp_xchg_type_names, },
  { "kernel_mode_names", &kernel_mode_names, },
  { "kernel_mode_stories", &kernel_mode_stories, },
  { "keyword_auth_names", &keyword_auth_names, },
  { "keyword_host_names", &keyword_host_names, },
  { "message_role_names", &message_role_names, },
  { "modecfg_attr_names", &modecfg_attr_names, },
  { "natt_method_names", &natt_method_names, },
  { "oakley_attr_bit_names", &oakley_attr_bit_names, },
  { "oakley_attr_names", &oakley_attr_names, },
  { "oakley_auth_names", &oakley_auth_names, },
  { "oakley_enc_names", &oakley_enc_names, },
  { "oakley_group_names", &oakley_group_names, },
  { "oakley_hash_names", &oakley_hash_names, },
  { "oakley_lifetime_names", &oakley_lifetime_names, },
  { "payload_names_ikev1orv2", &payload_names_ikev1orv2, },
  { "perspective_names", &perspective_names, },
  { "pluto_exit_code_names", &pluto_exit_code_names, },
  { "sa_lifetime_names", &sa_lifetime_names, },
  { "sa_role_names", &sa_role_names, },
  { "sa_type_names", &sa_type_names, },
  { "sd_action_names", &sd_action_names, },
  { "secret_kind_names", &secret_kind_names, },
  { "shunt_kind_names", &shunt_kind_names, },
  { "shunt_policy_names", &shunt_policy_names, },
  { "shunt_policy_percent_names", &shunt_policy_percent_names, },
  { "sit_bit_names", &sit_bit_names, },
  { "state_category_names", &state_category_names, },
  { "stf_status_names", &stf_status_names, },
  { "v1_notification_names", &v1_notification_names, },
  { "v2_notification_names", &v2_notification_names, },
  { "version_names", &version_names, },
  { "xauth_attr_names", &xauth_attr_names, },
#ifdef KERNEL_XFRM
  { "xfrm_policy_names", &xfrm_policy_names, },
#endif
  { NULL, NULL, }
};
